@@ -13,12 +13,15 @@
 # limitations under the License.
 
 import inspect
+import re
 import unittest
 
 import torch
 import torch.nn as nn
+from parameterized import parameterized
 from torch.fx import symbolic_trace
 
+from accelerate.big_modeling import attach_layerwise_casting_hooks
 from accelerate.hooks import (
     AlignDevicesHook,
     ModelHook,
@@ -28,7 +31,11 @@ from accelerate.hooks import (
     remove_hook_from_module,
     remove_hook_from_submodules,
 )
-from accelerate.test_utils import require_multi_gpu
+from accelerate.test_utils import require_multi_device, require_non_hpu, torch_device
+from accelerate.utils.constants import SUPPORTED_PYTORCH_LAYERS_FOR_UPCASTING
+
+
+torch_device = f"{torch_device}:0" if torch_device != "cpu" else "cpu"
 
 
 class ModelForTest(nn.Module):
@@ -53,21 +60,49 @@ class PostForwardHook(ModelHook):
 
 
 class HooksModelTester(unittest.TestCase):
+    def check_dtype_for_layerwise_upcasting(
+        self,
+        module,
+        storage_dtype,
+        loading_type,
+        patterns_to_check=None,
+    ):
+        for name, submodule in module.named_modules():
+            attrs = []
+            if getattr(submodule, "weight", None) is not None:
+                attrs.append(("weight", submodule.weight))
+            if getattr(submodule, "bias", None) is not None:
+                attrs.append(("bias", submodule.bias))
+
+            if not isinstance(submodule, SUPPORTED_PYTORCH_LAYERS_FOR_UPCASTING):
+                if patterns_to_check is None:
+                    for _, tensor in attrs:
+                        self.assertEqual(tensor.dtype, loading_type)
+                continue
+
+            if patterns_to_check and any(re.search(pat, name) for pat in patterns_to_check):
+                expected = loading_type
+            else:
+                expected = storage_dtype
+
+            for _, tensor in attrs:
+                self.assertEqual(tensor.dtype, expected)
+
     def test_add_and_remove_hooks(self):
         test_model = ModelForTest()
         test_hook = ModelHook()
 
         add_hook_to_module(test_model, test_hook)
-        self.assertEqual(test_model._hf_hook, test_hook)
-        self.assertTrue(hasattr(test_model, "_old_forward"))
+        assert test_model._hf_hook == test_hook
+        assert hasattr(test_model, "_old_forward")
 
         # Check adding the hook did not change the name or the signature
-        self.assertEqual(test_model.forward.__name__, "forward")
-        self.assertListEqual(list(inspect.signature(test_model.forward).parameters), ["x"])
+        assert test_model.forward.__name__ == "forward"
+        assert list(inspect.signature(test_model.forward).parameters) == ["x"]
 
         remove_hook_from_module(test_model)
-        self.assertFalse(hasattr(test_model, "_hf_hook"))
-        self.assertFalse(hasattr(test_model, "_old_forward"))
+        assert not hasattr(test_model, "_hf_hook")
+        assert not hasattr(test_model, "_old_forward")
 
     def test_append_and_remove_hooks(self):
         test_model = ModelForTest()
@@ -76,17 +111,17 @@ class HooksModelTester(unittest.TestCase):
         add_hook_to_module(test_model, test_hook)
         add_hook_to_module(test_model, test_hook, append=True)
 
-        self.assertEqual(isinstance(test_model._hf_hook, SequentialHook), True)
-        self.assertEqual(len(test_model._hf_hook.hooks), 2)
-        self.assertTrue(hasattr(test_model, "_old_forward"))
+        assert isinstance(test_model._hf_hook, SequentialHook) is True
+        assert len(test_model._hf_hook.hooks) == 2
+        assert hasattr(test_model, "_old_forward")
 
         # Check adding the hook did not change the name or the signature
-        self.assertEqual(test_model.forward.__name__, "forward")
-        self.assertListEqual(list(inspect.signature(test_model.forward).parameters), ["x"])
+        assert test_model.forward.__name__ == "forward"
+        assert list(inspect.signature(test_model.forward).parameters) == ["x"]
 
         remove_hook_from_module(test_model)
-        self.assertFalse(hasattr(test_model, "_hf_hook"))
-        self.assertFalse(hasattr(test_model, "_old_forward"))
+        assert not hasattr(test_model, "_hf_hook")
+        assert not hasattr(test_model, "_old_forward")
 
     def test_pre_forward_hook_is_executed(self):
         test_model = ModelForTest()
@@ -97,13 +132,13 @@ class HooksModelTester(unittest.TestCase):
         test_hook = PreForwardHook()
         add_hook_to_module(test_model, test_hook)
         output1 = test_model(x)
-        self.assertTrue(torch.allclose(output1, expected, atol=1e-5))
+        assert torch.allclose(output1, expected, atol=1e-5)
 
         # Attaching a hook to a model when it already has one replaces, does not chain
         test_hook = PreForwardHook()
         add_hook_to_module(test_model, test_hook)
         output1 = test_model(x)
-        self.assertTrue(torch.allclose(output1, expected, atol=1e-5))
+        assert torch.allclose(output1, expected, atol=1e-5)
 
         # You need to use the sequential hook to chain two or more hooks
         test_hook = SequentialHook(PreForwardHook(), PreForwardHook())
@@ -120,13 +155,13 @@ class HooksModelTester(unittest.TestCase):
         test_hook = PostForwardHook()
         add_hook_to_module(test_model, test_hook)
         output1 = test_model(x)
-        self.assertTrue(torch.allclose(output1, output + 1, atol=1e-5))
+        assert torch.allclose(output1, (output + 1), atol=1e-5)
 
         # Attaching a hook to a model when it already has one replaces, does not chain
         test_hook = PostForwardHook()
         add_hook_to_module(test_model, test_hook)
         output1 = test_model(x)
-        self.assertTrue(torch.allclose(output1, output + 1, atol=1e-5))
+        assert torch.allclose(output1, (output + 1), atol=1e-5)
 
         # You need to use the sequential hook to chain two or more hooks
         test_hook = SequentialHook(PostForwardHook(), PostForwardHook())
@@ -143,80 +178,81 @@ class HooksModelTester(unittest.TestCase):
         test_hook = PostForwardHook()
         add_hook_to_module(test_model, test_hook)
         output1 = test_model(x)
-        self.assertTrue(torch.allclose(output1, output + 1))
-        self.assertTrue(output1.requires_grad)
+        assert torch.allclose(output1, (output + 1))
+        assert output1.requires_grad
 
         test_hook.no_grad = True
         output1 = test_model(x)
-        self.assertFalse(output1.requires_grad)
+        assert not output1.requires_grad
 
-    @require_multi_gpu
+    @require_non_hpu  # hpu does not support device indexing "hpu:1"
+    @require_multi_device
     def test_align_devices_as_model_parallelism(self):
         model = ModelForTest()
         # Everything is on CPU
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # This will move each submodule on different devices
         add_hook_to_module(model.linear1, AlignDevicesHook(execution_device=0))
         add_hook_to_module(model.batchnorm, AlignDevicesHook(execution_device=0))
         add_hook_to_module(model.linear2, AlignDevicesHook(execution_device=1))
 
-        self.assertEqual(model.linear1.weight.device, torch.device(0))
-        self.assertEqual(model.batchnorm.weight.device, torch.device(0))
-        self.assertEqual(model.batchnorm.running_mean.device, torch.device(0))
-        self.assertEqual(model.linear2.weight.device, torch.device(1))
+        assert model.linear1.weight.device == torch.device(torch_device)
+        assert model.batchnorm.weight.device == torch.device(torch_device)
+        assert model.batchnorm.running_mean.device == torch.device(torch_device)
+        assert model.linear2.weight.device == torch.device(torch_device.replace(":0", ":1"))
 
         # We can still make a forward pass. The input does not need to be on any particular device
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, torch.device(1))
+        assert output.device == torch.device(torch_device.replace(":0", ":1"))
 
         # We can add a general hook to put back output on same device as input.
         add_hook_to_module(model, AlignDevicesHook(io_same_device=True))
-        x = torch.randn(2, 3).to(0)
+        x = torch.randn(2, 3).to(torch_device)
         output = model(x)
-        self.assertEqual(output.device, torch.device(0))
+        assert output.device == torch.device(torch_device)
 
     def test_align_devices_as_cpu_offload(self):
         model = ModelForTest()
 
         # Everything is on CPU
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # This will move each submodule on different devices
-        hook_kwargs = {"execution_device": 0 if torch.cuda.is_available() else "cpu", "offload": True}
+        hook_kwargs = {"execution_device": torch_device, "offload": True}
 
         add_hook_to_module(model.linear1, AlignDevicesHook(**hook_kwargs))
         add_hook_to_module(model.batchnorm, AlignDevicesHook(**hook_kwargs))
         add_hook_to_module(model.linear2, AlignDevicesHook(**hook_kwargs))
 
         # Parameters have been offloaded, so on the meta device
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
         # Buffers are not included in the offload by default, so are on the execution device
         device = torch.device(hook_kwargs["execution_device"])
-        self.assertEqual(model.batchnorm.running_mean.device, device)
+        assert model.batchnorm.running_mean.device == device
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_module(model.linear1)
         remove_hook_from_module(model.batchnorm)
         remove_hook_from_module(model.linear2)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # Now test with buffers included in the offload
         hook_kwargs = {
-            "execution_device": 0 if torch.cuda.is_available() else "cpu",
+            "execution_device": torch_device,
             "offload": True,
             "offload_buffers": True,
         }
@@ -226,103 +262,103 @@ class HooksModelTester(unittest.TestCase):
         add_hook_to_module(model.linear2, AlignDevicesHook(**hook_kwargs))
 
         # Parameters have been offloaded, so on the meta device, buffers included
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.running_mean.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
+        assert model.batchnorm.running_mean.device == torch.device("meta")
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_module(model.linear1)
         remove_hook_from_module(model.batchnorm)
         remove_hook_from_module(model.linear2)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
     def test_attach_align_device_hook_as_cpu_offload(self):
         model = ModelForTest()
 
         # Everything is on CPU
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # This will move each submodule on different devices
-        execution_device = 0 if torch.cuda.is_available() else "cpu"
+        execution_device = torch_device
         attach_align_device_hook(model, execution_device=execution_device, offload=True)
 
         # Parameters have been offloaded, so on the meta device
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
         # Buffers are not included in the offload by default, so are on the execution device
         device = torch.device(execution_device)
-        self.assertEqual(model.batchnorm.running_mean.device, device)
+        assert model.batchnorm.running_mean.device == device
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_submodules(model)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # Now test with buffers included in the offload
         attach_align_device_hook(model, execution_device=execution_device, offload=True, offload_buffers=True)
 
         # Parameters have been offloaded, so on the meta device, buffers included
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.running_mean.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
+        assert model.batchnorm.running_mean.device == torch.device("meta")
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_submodules(model)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
     def test_attach_align_device_hook_as_cpu_offload_with_weight_map(self):
         model = ModelForTest()
 
         # Everything is on CPU
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # This will move each submodule on different devices
-        execution_device = 0 if torch.cuda.is_available() else "cpu"
+        execution_device = torch_device
         attach_align_device_hook(
             model, execution_device=execution_device, offload=True, weights_map=model.state_dict()
         )
 
         # Parameters have been offloaded, so on the meta device
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
         # Buffers are not included in the offload by default, so are on the execution device
         device = torch.device(execution_device)
-        self.assertEqual(model.batchnorm.running_mean.device, device)
+        assert model.batchnorm.running_mean.device == device
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_submodules(model)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
         # Now test with buffers included in the offload
         attach_align_device_hook(
@@ -334,20 +370,20 @@ class HooksModelTester(unittest.TestCase):
         )
 
         # Parameters have been offloaded, so on the meta device, buffers included
-        self.assertEqual(model.linear1.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("meta"))
-        self.assertEqual(model.linear2.weight.device, torch.device("meta"))
-        self.assertEqual(model.batchnorm.running_mean.device, torch.device("meta"))
+        assert model.linear1.weight.device == torch.device("meta")
+        assert model.batchnorm.weight.device == torch.device("meta")
+        assert model.linear2.weight.device == torch.device("meta")
+        assert model.batchnorm.running_mean.device == torch.device("meta")
 
         x = torch.randn(2, 3)
         output = model(x)
-        self.assertEqual(output.device, device)
+        assert output.device == device
 
         # Removing hooks loads back the weights in the model.
         remove_hook_from_submodules(model)
-        self.assertEqual(model.linear1.weight.device, torch.device("cpu"))
-        self.assertEqual(model.batchnorm.weight.device, torch.device("cpu"))
-        self.assertEqual(model.linear2.weight.device, torch.device("cpu"))
+        assert model.linear1.weight.device == torch.device("cpu")
+        assert model.batchnorm.weight.device == torch.device("cpu")
+        assert model.linear2.weight.device == torch.device("cpu")
 
     def test_add_remove_hook_fx_graph_module(self):
         with torch.no_grad():
@@ -361,7 +397,7 @@ class HooksModelTester(unittest.TestCase):
 
             output2 = graph_model(x)
 
-            self.assertTrue(torch.allclose(output1, output2))
+            assert torch.allclose(output1, output2)
 
             add_hook_to_module(graph_model, test_hook)
             remove_hook_from_module(graph_model, recurse=True)
@@ -374,7 +410,7 @@ class HooksModelTester(unittest.TestCase):
             for node in graph_model.graph.nodes:
                 if node.name == "linear2":
                     linear2_node = node
-            self.assertTrue(linear2_node is not None)
+            assert linear2_node is not None
 
             graph_model.graph.inserting_after(linear2_node)
             new_node = graph_model.graph.create_node(
@@ -385,7 +421,7 @@ class HooksModelTester(unittest.TestCase):
             for node in graph_model.graph.nodes:
                 if node.name == "output":
                     output_node = node
-            self.assertTrue(output_node is not None)
+            assert output_node is not None
 
             output_node.replace_input_with(linear2_node, new_node)
 
@@ -395,4 +431,29 @@ class HooksModelTester(unittest.TestCase):
             output3 = graph_model(x)
 
             # Now the output is expected to be different since we modified the graph.
-            self.assertFalse(torch.allclose(output1, output3))
+            assert not torch.allclose(output1, output3)
+
+    @parameterized.expand(
+        [
+            (torch.float16, torch.float32),
+            (torch.float8_e4m3fn, torch.float32),
+            (torch.float8_e4m3fn, torch.float32, ["batchnorm"]),
+        ]
+    )
+    def test_layerwise_upcasting_inference(self, storage_dtype, compute_dtype, skip_modules_pattern=None):
+        test_model = ModelForTest()
+        loading_dtype = next(test_model.parameters()).data.dtype
+        inputs = torch.randn(2, 3)
+        inputs = inputs.to(compute_dtype) if inputs.dtype == torch.float32 else inputs
+
+        attach_layerwise_casting_hooks(
+            test_model,
+            storage_dtype=storage_dtype,
+            compute_dtype=compute_dtype,
+            skip_modules_pattern=skip_modules_pattern,
+        )
+        patterns_to_check = skip_modules_pattern if skip_modules_pattern else None
+        self.check_dtype_for_layerwise_upcasting(test_model, storage_dtype, loading_dtype, patterns_to_check)
+
+        with torch.no_grad():
+            _ = test_model(inputs)
