@@ -14,7 +14,6 @@
 
 import inspect
 import os
-import unittest
 from dataclasses import dataclass
 
 import torch
@@ -22,13 +21,23 @@ import torch
 from accelerate import Accelerator, DistributedDataParallelKwargs, GradScalerKwargs
 from accelerate.state import AcceleratorState
 from accelerate.test_utils import (
-    device_count,
+    DEFAULT_LAUNCH_COMMAND,
     execute_subprocess_async,
+    path_in_accelerate_package,
+    require_fp16,
     require_multi_device,
     require_non_cpu,
-    require_non_xpu,
+    run_first,
 )
-from accelerate.utils import AutocastKwargs, KwargsHandler, TorchDynamoPlugin, clear_environment
+from accelerate.test_utils.testing import AccelerateTestCase, slow
+from accelerate.utils import (
+    AutocastKwargs,
+    KwargsHandler,
+    ProfileKwargs,
+    TorchDynamoPlugin,
+    clear_environment,
+)
+from accelerate.utils.dataclasses import DistributedType
 
 
 @dataclass
@@ -38,38 +47,40 @@ class MockClass(KwargsHandler):
     c: float = 3.0
 
 
-class KwargsHandlerTester(unittest.TestCase):
+class KwargsHandlerTester(AccelerateTestCase):
     def test_kwargs_handler(self):
         # If no defaults are changed, `to_kwargs` returns an empty dict.
-        self.assertDictEqual(MockClass().to_kwargs(), {})
-        self.assertDictEqual(MockClass(a=2).to_kwargs(), {"a": 2})
-        self.assertDictEqual(MockClass(a=2, b=True).to_kwargs(), {"a": 2, "b": True})
-        self.assertDictEqual(MockClass(a=2, c=2.25).to_kwargs(), {"a": 2, "c": 2.25})
+        assert MockClass().to_kwargs() == {}
+        assert MockClass(a=2).to_kwargs() == {"a": 2}
+        assert MockClass(a=2, b=True).to_kwargs() == {"a": 2, "b": True}
+        assert MockClass(a=2, c=2.25).to_kwargs() == {"a": 2, "c": 2.25}
 
+    @require_fp16
     @require_non_cpu
-    @require_non_xpu
     def test_grad_scaler_kwargs(self):
         # If no defaults are changed, `to_kwargs` returns an empty dict.
         scaler_handler = GradScalerKwargs(init_scale=1024, growth_factor=2)
         AcceleratorState._reset_state()
         accelerator = Accelerator(mixed_precision="fp16", kwargs_handlers=[scaler_handler])
-        print(accelerator.use_fp16)
+        assert accelerator.mixed_precision == "fp16"
         scaler = accelerator.scaler
 
         # Check the kwargs have been applied
-        self.assertEqual(scaler._init_scale, 1024.0)
-        self.assertEqual(scaler._growth_factor, 2.0)
+        assert scaler._init_scale == 1024.0
+        assert scaler._growth_factor == 2.0
 
         # Check the other values are at the default
-        self.assertEqual(scaler._backoff_factor, 0.5)
-        self.assertEqual(scaler._growth_interval, 2000)
-        self.assertEqual(scaler._enabled, True)
+        assert scaler._backoff_factor == 0.5
+        assert scaler._growth_interval == 2000
+        assert scaler._enabled is True
 
+    @run_first
     @require_multi_device
     def test_ddp_kwargs(self):
-        cmd = ["torchrun", f"--nproc_per_node={device_count}", inspect.getfile(self.__class__)]
-        execute_subprocess_async(cmd, env=os.environ.copy())
+        cmd = DEFAULT_LAUNCH_COMMAND + [inspect.getfile(self.__class__)]
+        execute_subprocess_async(cmd)
 
+    @require_fp16
     @require_non_cpu
     def test_autocast_kwargs(self):
         kwargs = AutocastKwargs(enabled=False)
@@ -94,6 +105,52 @@ class KwargsHandlerTester(unittest.TestCase):
             # We should be back in fp16
             assert g_float16.dtype == torch.float16
 
+    @slow
+    def test_profile_kwargs(self):
+        # Arrange
+        schedule_options = [
+            dict(wait=1, warmup=1, active=2, repeat=1),
+            dict(wait=2, warmup=2, active=2, repeat=2),
+            dict(wait=0, warmup=1, active=3, repeat=3, skip_first=1),
+            dict(wait=3, warmup=2, active=1, repeat=1, skip_first=2),
+            dict(wait=1, warmup=0, active=1, repeat=5),
+        ]
+
+        total_steps = 100
+
+        for option in schedule_options:
+            count = 0
+            table_outputs = []
+            steps_per_cycle = option["wait"] + option["warmup"] + option["active"]
+            effective_steps = max(0, total_steps - option.get("skip_first", 0))
+            cycles = effective_steps // steps_per_cycle
+            if option["repeat"] > 0:
+                expected_count = min(cycles, option["repeat"])
+            else:
+                expected_count = cycles
+
+            def on_trace_ready(prof):
+                nonlocal count
+                nonlocal table_outputs
+
+                count += 1
+                table_outputs.append(prof.key_averages().table(sort_by="cpu_time_total", row_limit=-1))
+
+            kwargs = ProfileKwargs(activities=["cpu"], on_trace_ready=on_trace_ready, schedule_option=option)
+            accelerator = Accelerator(kwargs_handlers=[kwargs])
+
+            # Act
+            with accelerator.profile() as prof:
+                for _ in range(total_steps):
+                    prof.step()
+                    torch.tensor([1, 2, 3, 4, 5], device=accelerator.device)
+
+            # Assert
+            assert isinstance(prof, torch.profiler.profile)
+            assert count == expected_count, f"Option: {option}, Expected count: {expected_count}, but got {count}"
+            for output in table_outputs:
+                self.assertIn("CPU time total:", output)
+
     def test_torch_dynamo_plugin(self):
         with clear_environment():
             prefix = "ACCELERATE_DYNAMO_"
@@ -103,12 +160,24 @@ class KwargsHandlerTester(unittest.TestCase):
             os.environ[prefix + "MODE"] = "reduce-overhead"
 
             dynamo_plugin_kwargs = TorchDynamoPlugin().to_kwargs()
-            self.assertEqual(dynamo_plugin_kwargs, {"backend": "aot_ts_nvfuser", "mode": "reduce-overhead"})
+            assert dynamo_plugin_kwargs == {"backend": "aot_ts_nvfuser", "mode": "reduce-overhead"}
+        assert os.environ.get(prefix + "BACKEND") != "aot_ts_nvfuser"
+
+    @run_first
+    @require_multi_device
+    def test_ddp_comm_hook(self):
+        cmd = DEFAULT_LAUNCH_COMMAND + [path_in_accelerate_package("test_utils", "scripts", "test_ddp_comm_hook.py")]
+        execute_subprocess_async(cmd)
 
 
-if __name__ == "__main__":
+def main():
     ddp_scaler = DistributedDataParallelKwargs(bucket_cap_mb=15, find_unused_parameters=True)
     accelerator = Accelerator(kwargs_handlers=[ddp_scaler])
+
+    # Skip this test due to TorchXLA not using torch.nn.parallel.DistributedDataParallel for model wrapping.
+    if accelerator.distributed_type == DistributedType.XLA:
+        return
+
     model = torch.nn.Linear(100, 200)
     model = accelerator.prepare(model)
 
@@ -131,3 +200,7 @@ if __name__ == "__main__":
     # Raise error at the end to make sure we don't stop at the first failure.
     if len(error_msg) > 0:
         raise ValueError(error_msg)
+
+
+if __name__ == "__main__":
+    main()
