@@ -16,7 +16,12 @@ import unittest
 
 from torch import nn
 
-from accelerate.test_utils import memory_allocated_func, require_non_cpu, torch_device
+from accelerate.test_utils import (
+    memory_allocated_func,
+    require_non_cpu,
+    require_non_torch_xla,
+    torch_device,
+)
 from accelerate.utils.memory import find_executable_batch_size, release_memory
 
 
@@ -35,6 +40,15 @@ class ModelForTest(nn.Module):
         return self.linear2(self.batchnorm(self.linear1(x)))
 
 
+class BigModelForTest(ModelForTest):
+    def __init__(self):
+        super().__init__()
+        self.linear3 = nn.Linear(5, 1000)
+
+    def forward(self, x):
+        return self.linear3(super().forward(x))
+
+
 class MemoryTest(unittest.TestCase):
     def test_memory_implicit(self):
         batch_sizes = []
@@ -47,7 +61,31 @@ class MemoryTest(unittest.TestCase):
                 raise_fake_out_of_memory()
 
         mock_training_loop_function()
-        self.assertListEqual(batch_sizes, [128, 64, 32, 16, 8])
+        assert batch_sizes == [
+            128,
+            115,
+            103,
+            92,
+            82,
+            73,
+            65,
+            58,
+            52,
+            46,
+            41,
+            36,
+            32,
+            28,
+            25,
+            22,
+            19,
+            17,
+            15,
+            13,
+            11,
+            9,
+            8,
+        ]
 
     def test_memory_explicit(self):
         batch_sizes = []
@@ -61,8 +99,32 @@ class MemoryTest(unittest.TestCase):
             return batch_size, arg1
 
         bs, arg1 = mock_training_loop_function("hello")
-        self.assertListEqual(batch_sizes, [128, 64, 32, 16, 8])
-        self.assertListEqual([bs, arg1], [8, "hello"])
+        assert batch_sizes == [
+            128,
+            115,
+            103,
+            92,
+            82,
+            73,
+            65,
+            58,
+            52,
+            46,
+            41,
+            36,
+            32,
+            28,
+            25,
+            22,
+            19,
+            17,
+            15,
+            13,
+            11,
+            9,
+            8,
+        ]
+        assert [bs, arg1] == [8, "hello"]
 
     def test_start_zero(self):
         @find_executable_batch_size(starting_batch_size=0)
@@ -71,7 +133,7 @@ class MemoryTest(unittest.TestCase):
 
         with self.assertRaises(RuntimeError) as cm:
             mock_training_loop_function()
-            self.assertIn("No executable batch size found, reached zero.", cm.exception.args[0])
+            assert "No executable batch size found, reached zero." in cm.exception.args[0]
 
     def test_approach_zero(self):
         @find_executable_batch_size(starting_batch_size=16)
@@ -82,7 +144,7 @@ class MemoryTest(unittest.TestCase):
 
         with self.assertRaises(RuntimeError) as cm:
             mock_training_loop_function()
-            self.assertIn("No executable batch size found, reached zero.", cm.exception.args[0])
+            assert "No executable batch size found, reached zero." in cm.exception.args[0]
 
     def test_verbose_guard(self):
         @find_executable_batch_size(starting_batch_size=128)
@@ -92,8 +154,8 @@ class MemoryTest(unittest.TestCase):
 
         with self.assertRaises(TypeError) as cm:
             mock_training_loop_function(128, "hello", "world")
-            self.assertIn("Batch size was passed into `f`", cm.exception.args[0])
-            self.assertIn("`f(arg1='hello', arg2='world')", cm.exception.args[0])
+            assert "Batch size was passed into `f`" in cm.exception.args[0]
+            assert "`f(arg1='hello', arg2='world')" in cm.exception.args[0]
 
     def test_any_other_error(self):
         @find_executable_batch_size(starting_batch_size=16)
@@ -102,13 +164,21 @@ class MemoryTest(unittest.TestCase):
 
         with self.assertRaises(ValueError) as cm:
             mock_training_loop_function()
-            self.assertIn("Oops, we had an error!", cm.exception.args[0])
+            assert "Oops, we had an error!" in cm.exception.args[0]
 
     @require_non_cpu
+    @require_non_torch_xla
     def test_release_memory(self):
         starting_memory = memory_allocated_func()
-        model = ModelForTest()
+
+        if torch_device.startswith("hpu"):
+            # hpu has a minimum memory allocation that cannot be released,
+            # we need to surpass it by using a bigger model (>5767296 bytes)
+            model = BigModelForTest()
+        else:
+            model = ModelForTest()
+
         model.to(torch_device)
-        self.assertGreater(memory_allocated_func(), starting_memory)
+        assert memory_allocated_func() > starting_memory
         model = release_memory(model)
-        self.assertEqual(memory_allocated_func(), starting_memory)
+        assert memory_allocated_func() == starting_memory
